package METEOR;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::IQXML;
use IQ::COMMON;
use File::ReadBackwards;

use vars qw($MTREXT $rMETEOR $TMETEOR);

$METEOR::MTREXT = "MTR";
$METEOR::TMETEOR = "METEOR/meteor.pl";                # version 0.4.3
$METEOR::rMETEOR = { "$METEOR::MTREXT-exact" => 1, "$METEOR::MTREXT-stem" => 1, "$METEOR::MTREXT-wnstm" => 1, "$METEOR::MTREXT-wnsyn" => 1 };
$METEOR::lMETEOR = ( "$METEOR::MTREXT-exact", "$METEOR::MTREXT-stem", "$METEOR::MTREXT-wnstm", "$METEOR::MTREXT-wnsyn");

sub read_meteor {
   #description _ read METEOR value from report file
   #param1  _ report filename
   #@return _ METEOR score

   my $report = shift;

   my $REPORT = File::ReadBackwards->new($report) or die "Couldn't open input file: $report\n";
   my $line = $REPORT->readline;
   while (!($line =~ /^Score:.*/)) { $line = $REPORT->readline; }
   $REPORT->close();
   chomp($line);
   my @lmeteor = split(" ", $line);

   return $lmeteor[1];
}

sub read_meteor_segments {
   #description _ read METEOR value from report file (for all segments)
   #param1  _ report filename
   #@return _ METEOR score list

   my $report = shift;

   open(AUX, "< $report ") or die "couldn't open file: $report\n";

   my @lMTR;
   my $index = 0;
   while (my $aux = <AUX>) {
      if ($aux =~ /^Document:.*/) {
         chomp($aux);
         my @laux = split(/ +/, $aux);
         $index = $laux[3];
      }
      elsif ($aux =~ /^sgmScore:.*/) {
         my @laux = split(/ +/, $aux);
         $lMTR[$index-1] = $laux[1];
      }
   }
   close(AUX);

   return \@lMTR;
}

sub computeMultiMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (multiple references)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [hash reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ mode (0:exact | 1:+stem | 2:+wnstem | 3:+wnsyn)
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $mode = shift;
   my $verbose = shift;

   my $toolMETEOR = "perl $tools/$METEOR::TMETEOR";

   my $refMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REFEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $refMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_multidoc($Href, $refMTRsgml, 2, $verbose); }
   my $srcMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SRCEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $srcMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_doc($src, $srcMTRsgml, 0, $verbose); }
   my $outMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SYSEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $outMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_doc($out, $outMTRsgml, 1, $verbose); }

   my $reportMTR = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REPORTEXT";
   if ($verbose > 1) { print STDERR "building $reportMTR...\n"; }
   my $modules;
   if ($mode == 0) { $modules = " --modules \"exact\""; }
   elsif ($mode == 1) { $modules = " --modules \"exact porter_stem\""; }
   elsif ($mode == 2) { $modules = " --modules \"exact porter_stem wn_stem\""; }
   elsif ($mode == 3) { $modules = " --modules \"exact porter_stem wn_stem wn_synonymy\""; }
   else { die "[MTR] UNKNOWN MODE <$mode>!!!\n"; }

   system("$toolMETEOR -s dummysysTST -t $outMTRsgml -r $refMTRsgml $modules > $reportMTR");
   my $SYS = read_meteor($reportMTR);
   my $SEG = read_meteor_segments($reportMTR);
   system("rm $reportMTR");
   if (-e $srcMTRsgml) { system "rm $srcMTRsgml"; }
   if (-e $refMTRsgml) { system "rm $refMTRsgml"; }
   if (-e $outMTRsgml) { system "rm $outMTRsgml"; }

   return ($SYS, $SEG);
}

sub doMultiMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (multiple references)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file(s) [hash reference]
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mMETEOR = keys %{$METEOR::rMETEOR};
   while (($i < scalar(@mMETEOR)) and (!$GO)) { if (exists($M->{$mMETEOR[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "METEOR.."; }
      my $reportMTRexactXML = "./$TGTname/$REFname/$MTREXT-exact.$COMMON::XMLEXT";
      my $reportMTRstemXML = "./$TGTname/$REFname/$MTREXT-stem.$COMMON::XMLEXT";
      my $reportMTRwnstmXML = "./$TGTname/$REFname/$MTREXT-wnstm.$COMMON::XMLEXT";
      my $reportMTRwnsynXML = "./$TGTname/$REFname/$MTREXT-wnsyn.$COMMON::XMLEXT";

      if (((!(-e $reportMTRexactXML) and !(-e $reportMTRexactXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-exact"}) { #exact
         my ($SYS, $SEG) = METEOR::computeMultiMETEOR($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, 0, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-exact", $SYS, $SEG, $reportMTRexactXML, $verbose);
      }
      if (((!(-e $reportMTRstemXML) and !(-e $reportMTRstemXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-stem"}) { #exact + porter
         my ($SYS, $SEG) = METEOR::computeMultiMETEOR($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, 1, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-stem", $SYS, $SEG, $reportMTRstemXML, $verbose);
      }
      if (((!(-e $reportMTRwnstmXML) and !(-e $reportMTRwnstmXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-wnstm"}) { #exact + porter_stem + wn_stem
         my ($SYS, $SEG) = METEOR::computeMultiMETEOR($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, 2, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-wnstm", $SYS, $SEG, $reportMTRwnstmXML, $verbose);
      }
      if (((!(-e $reportMTRwnsynXML) and !(-e $reportMTRwnsynXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-wnsyn"}) { #exact + porter_stem + wn_stem + wn_syn
         my ($SYS, $SEG) = METEOR::computeMultiMETEOR($src, $out, $Href, $remakeREPORTS, $TGTname, $REFname, $tools, 3, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-wnsyn", $SYS, $SEG, $reportMTRwnsynXML, $verbose);
      }
   }
}

sub computeMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (single reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ mode (0:exact | 1:+stem | 2:+wnstem | 3:+wnsyn)
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $mode = shift;
   my $verbose = shift;

   my $toolMETEOR = "perl $tools/$METEOR::TMETEOR";

   my $refMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REFEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $refMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_doc($ref, $refMTRsgml, 2, $verbose); }
   my $srcMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SRCEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $srcMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_doc($src, $srcMTRsgml, 0, $verbose); }
   my $outMTRsgml = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::SYSEXT.$MTREXT.$COMMON::SGMLEXT";
   if ((!(-e $outMTRsgml)) or $remakeREPORTS) { IQXML::MTR_f_create_mteval_doc($out, $outMTRsgml, 1, $verbose); }

   my $reportMTR = "./$COMMON::TMP/".rand($COMMON::NRAND).".$COMMON::REPORTEXT";
   if ($verbose > 1) { print STDERR "building $reportMTR...\n"; }
   my $modules;
   if ($mode == 0) { $modules = " --modules \"exact\""; }
   elsif ($mode == 1) { $modules = " --modules \"exact porter_stem\""; }
   elsif ($mode == 2) { $modules = " --modules \"exact porter_stem wn_stem\""; }
   elsif ($mode == 3) { $modules = " --modules \"exact porter_stem wn_stem wn_synonymy\""; }
   else { die "[MTR] UNKNOWN MODE <$mode>!!!\n"; }

   system("$toolMETEOR -s dummysysTST -t $outMTRsgml -r $refMTRsgml $modules > $reportMTR");
   my $SYS = read_meteor($reportMTR);
   my $SEG = read_meteor_segments($reportMTR);
   system("rm $reportMTR");
   if (-e $srcMTRsgml) { system "rm $srcMTRsgml"; }
   if (-e $refMTRsgml) { system "rm $refMTRsgml"; }
   if (-e $outMTRsgml) { system "rm $outMTRsgml"; }

   return ($SYS, $SEG);
}

sub doMETEOR {
   #description _ computes METEOR scores (exact + stem + wnstem + wnsyn) (single reference)
   #param1 _ source file
   #param2 _ candidate file
   #param3 _ reference file
   #param4 _ remake reports? (1 - yes :: 0 - no)
   #param5 _ TARGET NAME (base filename)
   #param6 _ REF NAME (base filename)
   #param7 _ tools
   #param8 _ metrics to compute
   #param9 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGTname = shift;
   my $REFname = shift;
   my $tools = shift;
   my $M = shift;
   my $verbose = shift;

   my $GO = 0; my $i = 0;
   my @mMETEOR = keys %{$METEOR::rMETEOR};
   while (($i < scalar(@mMETEOR)) and (!$GO)) { if (exists($M->{$mMETEOR[$i]})) { $GO = 1; } $i++; }

   if ($GO) {
      if ($verbose == 1) { print STDERR "METEOR.."; }
      my $reportMTRexactXML = "./$TGTname/$REFname/$MTREXT-exact.$COMMON::XMLEXT";
      my $reportMTRstemXML = "./$TGTname/$REFname/$MTREXT-stem.$COMMON::XMLEXT";
      my $reportMTRwnstmXML = "./$TGTname/$REFname/$MTREXT-wnstm.$COMMON::XMLEXT";
      my $reportMTRwnsynXML = "./$TGTname/$REFname/$MTREXT-wnsyn.$COMMON::XMLEXT";

      if (((!(-e $reportMTRexactXML) and !(-e $reportMTRexactXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-exact"}) { #exact
         my ($SYS, $SEG) = METEOR::computeMETEOR($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, 0, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-exact", $SYS, $SEG, $reportMTRexactXML, $verbose);
      }
      if (((!(-e $reportMTRstemXML) and !(-e $reportMTRstemXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-stem"}) { #exact + porter
         my ($SYS, $SEG) = METEOR::computeMETEOR($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, 1, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-stem", $SYS, $SEG, $reportMTRstemXML, $verbose);
      }
      if (((!(-e $reportMTRwnstmXML) and !(-e $reportMTRwnstmXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-wnstm"}) { #exact + porter_stem + wn_stem
         my ($SYS, $SEG) = METEOR::computeMETEOR($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, 2, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-wnstm", $SYS, $SEG, $reportMTRwnstmXML, $verbose);
      }
      if (((!(-e $reportMTRwnsynXML) and !(-e $reportMTRwnsynXML.".$COMMON::GZEXT")) or $remakeREPORTS) and $M->{"$MTREXT-wnsyn"}) { #exact + porter_stem + wn_stem + wn_syn
         my ($SYS, $SEG) = METEOR::computeMETEOR($src, $out, $ref, $remakeREPORTS, $TGTname, $REFname, $tools, 3, $verbose);
         IQXML::write_report($TGTname, $REFname, "$MTREXT-wnsyn", $SYS, $SEG, $reportMTRwnsynXML, $verbose);
      }
   }
}

1;
